# -*- coding: utf8 -*-

from django.db import models
from django.core.exceptions import SuspiciousOperation
from django.template.defaultfilters import slugify

from portailuser.models import PortailUser
from gestion import fields

from datetime import date, timedelta


def _sondage_date_fin_default():
    """
    Date de fin par défaut d’un sondage
    """

    return date.today() + timedelta(days=14)


class Sondage(models.Model):
    """Modèle du sondage"""

    slug = models.SlugField(max_length=64, editable=False, unique=True)
    auteur = models.ForeignKey(
        PortailUser,
        editable=False,
        related_name="sondage_auteur_set",
        on_delete=models.CASCADE,
    )
    est_demarre = models.BooleanField(editable=False, default=False)
    supprime = models.BooleanField(editable=False, default=False)

    titre = models.CharField(verbose_name="Titre du sondage", max_length=64)
    description = models.TextField(
        verbose_name="Description",
        help_text="Quel est l'objectif de ce sondage ?",
        blank=True,
    )

    date_debut = fields.DateField(null=True, blank=True, editable=False)
    date_fin = fields.DateField(verbose_name="Date de fin du sondage", default=_sondage_date_fin_default)
    choix_multiple = models.BooleanField(verbose_name="Autoriser les choix multiples", default=False)
    resultats_masques = models.BooleanField(verbose_name="Résultats masqués", default=False)
    anonyme = models.BooleanField(verbose_name="Votes anonymes et non modifiables", default=False)

    votants = models.ManyToManyField(PortailUser, editable=False, through="Voix")

    def __str__(self):
        return self.titre

    def ajouter_votant(self, utilisateur):
        """Ajoute ``utilisateur`` comme votant au sondage.
        Renvoie None si ``utilisateur`` a déjà participé au sondage.

        Erreur
        ------
        SuspiciousOperation
            Si le sondage a déjà démarré.
        """
        if self.est_demarre:
            raise SuspiciousOperation("Le sondage est démarré")

        if Voix.objects.filter(sondage=self, votant=utilisateur):
            return

        # Ne pas oublier de sauver la voix :-'
        Voix(sondage=self, votant=utilisateur).save()

    def supprimer_votants(self):
        if self.est_demarre:
            raise SuspiciousOperation("Le sondage est démarré")
        Voix.objects.filter(sondage=self).delete()

    def get_choix(self):
        return [i.nom for i in self.choix_set.all()]

    def set_choix(self, choix):
        """
        Erreur
        ------
        SuspiciousOperation
            Si le sondage a déjà démarré.
        """
        if self.est_demarre:
            raise SuspiciousOperation("Le sondage est démarré")
        self.choix_set.all().delete()
        for nom in choix:
            Choix(sondage=self, nom=nom).save()

    def set_vote(self, utilisateur, vote):
        voix = Voix.objects.get(sondage=self, votant=utilisateur)
        assert voix
        voix.choix.clear()
        for choix in self.choix_set.all():
            if str(choix.id) in vote:
                if self.anonyme:
                    choix.votes += 1
                    choix.save()
                else:
                    voix.choix.add(choix)
        voix.exprimee = True
        voix.save()

    def save(self, **kwargs):
        if not self.slug:
            slug = slugify(self.titre)[0:60]
            if Sondage.objects.filter(slug=slug).count() > 0:
                i = 2
                while Sondage.objects.filter(slug=slug + str(i)).count() > 0:
                    i += 1
                slug = slug + str(i)
            self.slug = slug

        super(Sondage, self).save(**kwargs)


class Choix(models.Model):
    """Modèle pour les différents choix d'un sondage"""

    sondage = models.ForeignKey(Sondage, on_delete=models.CASCADE)
    nom = models.CharField(verbose_name="Nom", max_length=64)
    votes = models.IntegerField(default=0)  # Utilisé uniquement pour les votes anonymes

    def __str__(self):
        return self.nom

    class Meta:
        unique_together = (("sondage", "nom"),)
        ordering = ["id"]


class Voix(models.Model):
    """Modèle pour les voix des participants à un sondage"""

    sondage = models.ForeignKey(Sondage, on_delete=models.CASCADE)
    votant = models.ForeignKey(PortailUser, on_delete=models.CASCADE)
    exprimee = models.BooleanField(default=False)
    choix = models.ManyToManyField(Choix)

    class Meta:
        unique_together = (("sondage", "votant"),)
